#include <is_i2cdma.h>

CyU3PReturnStatus_t I2cDmaInit(void)
{
	CyU3PReturnStatus_t status = CY_U3P_SUCCESS;
	CyU3PI2cConfig_t config;

	status = CyU3PI2cInit();

	if(status != CY_U3P_SUCCESS)
	{
		return status;
	}

    g_bI2cLock = FALSE;
    g_bI2cDmaBusy = FALSE;

    status = I2cDmaCreateDmaChannel();
	if(status != CY_U3P_SUCCESS)
	{
		return status;
	}

	g_DmaBufferWrite.buffer = &g_u8WriteDataDma[0];
	g_DmaBufferWrite.count  = 0;
	g_DmaBufferWrite.size   = I2C_DMA_BUF_SIZE;
	g_DmaBufferWrite.status = NULL;

	g_DmaBufferRead.buffer  = &g_u8ReadDataDma[0];
	g_DmaBufferRead.count   = 0;
	g_DmaBufferRead.size    = I2C_DMA_BUF_SIZE;
	g_DmaBufferRead.status  = NULL;


	config.bitRate = 400000; // 400 kHz
	config.isDma = TRUE;  // Register transfer mode
	config.busTimeout = 0xFFFFFFFFU; // no timeout
	config.dmaTimeout = 0xFFFF; // no timeout

	status = CyU3PI2cSetConfig(&config, &I2cDmaIR);

	if(status != CY_U3P_SUCCESS)
	{
		return status;
	}

	return status;
}

void I2cDmaIR(CyU3PI2cEvt_t evt, CyU3PI2cError_t error)
{
	CyU3PReturnStatus_t status = CY_U3P_SUCCESS;
	switch(evt)
	{
	case CY_U3P_I2C_EVENT_RX_DONE:
	case CY_U3P_I2C_EVENT_TX_DONE:
		if(g_bI2cLock == CyTrue)
		{
            /* FIX: Wait for DMA callback and then reset the channel. */
            while (g_bI2cDmaBusy == TRUE);

			status = CyU3PDmaChannelReset(&g_HandleI2cDmaCh);
			g_bI2cLock = FALSE;
		}
		else
		{
			I2cDmaErrorEvent(error);
		}
		break;
	case CY_U3P_I2C_EVENT_TIMEOUT:
		break;
	case CY_U3P_I2C_EVENT_LOST_ARBITRATION:
		break;
	case CY_U3P_I2C_EVENT_ERROR:
		I2cDmaErrorEvent(error);
		break;
	default:
		break;
	}
}

void I2cDmaErrorEvent(CyU3PI2cError_t error)
{
	switch(error)
	{
	case CY_U3P_I2C_ERROR_NAK_BYTE_0:
		break;
	case CY_U3P_I2C_ERROR_NAK_BYTE_1:
		break;
	case CY_U3P_I2C_ERROR_NAK_BYTE_2:
		break;
	case CY_U3P_I2C_ERROR_NAK_BYTE_3:
		break;
	case CY_U3P_I2C_ERROR_NAK_BYTE_4:
		break;
	case CY_U3P_I2C_ERROR_NAK_BYTE_5:
		break;
	case CY_U3P_I2C_ERROR_NAK_BYTE_6:
		break;
	case CY_U3P_I2C_ERROR_NAK_BYTE_7:
		break;
	case CY_U3P_I2C_ERROR_NAK_DATA:
		break;
	case CY_U3P_I2C_ERROR_PREAMBLE_EXIT_NACK_ACK:
		break;
	case CY_U3P_I2C_ERROR_PREAMBLE_EXIT:
		break;
	case CY_U3P_I2C_ERROR_NAK_TX_UNDERFLOW:
		break;
	case CY_U3P_I2C_ERROR_NAK_TX_OVERFLOW:
		break;
	case CY_U3P_I2C_ERROR_NAK_RX_UNDERFLOW:
		break;
	case CY_U3P_I2C_ERROR_NAK_RX_OVERFLOW:
		break;
	default:
		break;
	}
}
CyU3PReturnStatus_t I2cDmaWrite(uint8_t u8DvcAdr, uint8_t u8SizeAdr, uint8_t *pu8Adr, uint8_t u8SizeData, uint8_t *pu8Data)
{
	while(g_bI2cLock==CyTrue);

	CyU3PReturnStatus_t status = CY_U3P_SUCCESS;

	g_DmaBufferWrite.buffer = pu8Data;
	g_DmaBufferWrite.count = u8SizeData;

	status = CyU3PDmaChannelSetupSendBuffer(&g_HandleI2cDmaCh, &g_DmaBufferWrite);

    /* FIX: Make sure to check status of DMA call. */
    if(status == CY_U3P_SUCCESS)
	{
		g_bI2cLock = TRUE;
        /* FIX: Mark DMA channel busy. */
		g_bI2cDmaBusy = TRUE;
		status = I2cDmaCommunicate(FALSE, u8DvcAdr, u8SizeAdr, pu8Adr, u8SizeData, pu8Data);
	}
	return status;
}
CyU3PReturnStatus_t I2cDmaRead(uint8_t u8DvcAdr, uint8_t u8SizeAdr, uint8_t *pu8Adr, uint8_t u8SizeData, uint8_t *pu8Data)
{
	while(g_bI2cLock==CyTrue);

	CyU3PReturnStatus_t status = CY_U3P_SUCCESS;

	g_DmaBufferRead.buffer = pu8Data;
	g_DmaBufferRead.count = u8SizeData;

	status = CyU3PDmaChannelSetupRecvBuffer(&g_HandleI2cDmaCh, &g_DmaBufferRead);
    /* FIX: Make sure to check status of DMA call. */
    if(status == CY_U3P_SUCCESS)
	{
		g_bI2cLock = TRUE;
        /* FIX: Mark DMA channel busy. */
		g_bI2cDmaBusy = TRUE;
        status = I2cDmaCommunicate(TRUE, u8DvcAdr, u8SizeAdr, pu8Adr, u8SizeData, pu8Data);
	}
	return status;
}
CyU3PReturnStatus_t I2cDmaCommunicate(CyBool_t bIsRead, uint8_t u8DvcAdr, uint8_t u8SizeAdr, uint8_t *pu8Adr, uint8_t u8SizeData, uint8_t *pu8Data)
{
	CyU3PReturnStatus_t status = CY_U3P_SUCCESS;
	CyU3PI2cPreamble_t preamble;
	uint8_t CntBytesPreamble = 0;
	uint8_t LengthAdr = 0;
	uint8_t u8DvcAdrWrite = u8DvcAdr & 0xFE;
	uint8_t u8DvcAdrRead = u8DvcAdr | 0x01;

	preamble.ctrlMask = 0;
	preamble.length = 0;

	if(u8SizeAdr > 0)
	{
		preamble.buffer[0] = u8DvcAdrWrite;
		CntBytesPreamble++;
	}

	for(LengthAdr = 0; LengthAdr < u8SizeAdr; LengthAdr++)
	{
		preamble.buffer[LengthAdr+1] = pu8Adr[LengthAdr];
		CntBytesPreamble++;
	}

	if(bIsRead == CyTrue)
	{
		if(u8SizeAdr > 0)
		{
			preamble.buffer[++LengthAdr] = u8DvcAdrRead;
			preamble.length = (++CntBytesPreamble);
			preamble.ctrlMask = 0x01 << (CntBytesPreamble-2);
		}
	}
	else
	{
		preamble.length = CntBytesPreamble;
		preamble.ctrlMask = 0x00;
	}

	status = CyU3PI2cSendCommand(&preamble, u8SizeData, bIsRead);

	return status;
}

CyU3PReturnStatus_t I2cDmaCreateDmaChannel(void)
{
    CyU3PDmaChannelConfig_t dmaI2cConfig;
    CyU3PReturnStatus_t status = CY_U3P_SUCCESS;


    /* Create a DMA MANUAL Channel between two sockets of the U port */
    dmaI2cConfig.size           = I2C_DMA_BUF_SIZE;
    dmaI2cConfig.count          = I2C_DMA_BUF_COUNT;
    dmaI2cConfig.prodSckId      = CY_U3P_LPP_SOCKET_I2C_PROD;
    dmaI2cConfig.consSckId      = CY_U3P_LPP_SOCKET_I2C_CONS;
    dmaI2cConfig.dmaMode        = CY_U3P_DMA_MODE_BYTE;
    /* FIX: Changed list of DMA notifications required. */
    dmaI2cConfig.notification   = CY_U3P_DMA_CB_SEND_CPLT | CY_U3P_DMA_CB_RECV_CPLT;
    dmaI2cConfig.cb             = I2cDmaCallbackDma;
    dmaI2cConfig.prodHeader     = 0;
    dmaI2cConfig.prodFooter     = 0;
    dmaI2cConfig.consHeader     = 0;
    dmaI2cConfig.prodAvailCount = 0;

    /* Create the channel */
    status = CyU3PDmaChannelCreate(&g_HandleI2cDmaCh, CY_U3P_DMA_TYPE_AUTO, &dmaI2cConfig);
    return status;
}

/* Callback funtion for the DMA event notification */
void I2cDmaCallbackDma(CyU3PDmaChannel *chHandle, CyU3PDmaCbType_t type, CyU3PDmaCBInput_t *input)
{
    /* If this is a callback notifying that the SetupSend or SetupRecv is complete,
       mark the DMA channel as not busy. */
    if ((type == CY_U3P_DMA_CB_SEND_CPLT) || (type == CY_U3P_DMA_CB_RECV_CPLT))
    {
    	g_bI2cDmaBusy = FALSE;
    }
}
